/**
 * OW2 Util
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: CurrentInvocationID.java 5465 2010-04-21 12:31:45Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.auditreport.impl;

import org.ow2.util.auditreport.api.IAuditID;
import org.ow2.util.auditreport.api.ICurrentInvocationID;

/**
 * Allows to get the current ID.
 * @author Florent Benoit
 */
public class CurrentInvocationID implements ICurrentInvocationID {

    /**
     * Inherited Local thread used to keep the ID.
     */
    private ThreadLocal<IAuditID> threadLocal = null;

    /**
     * Default constructor.
     */
    public CurrentInvocationID() {
        this.threadLocal = new ThreadLocal<IAuditID>();
    }

    /**
     * Unique instance of this current object.
     */
    private static ICurrentInvocationID unique = null;

    /**
     * Return the unique instance of this object.
     * @return SecurityCurrent return the current
     */
    public static synchronized ICurrentInvocationID getInstance() {
        if (unique == null) {
            // Build a default implementation
            unique = new CurrentInvocationID();
        }
        return unique;
    }

    /**
     * Associates the given audit ID to the current thread.
     * @param auditID the ID to associate to the current thread.
     * @return the previous ID
     */
    public IAuditID setAuditID(final IAuditID auditID) {
        IAuditID previous = threadLocal.get();
        threadLocal.set(auditID);
        return previous;
    }

    /**
     * Gets the current audit ID (if any).
     * @return the ID to associate to the current thread
     */
    public IAuditID getAuditID() {
        return threadLocal.get();
    }

    /**
     * Initialize a new Audit ID with an optional value.
     * @param id the initial value
     */
    public void init(final IAuditID id) {
        IAuditID newId = null;
        if (id != null) {
            newId = id;
        } else {
            newId = new AuditIDImpl();
            newId.generate();
        }
        threadLocal.set(newId);
    }

    /**
     * Generate a new ID and return the previously set.
     * @return the previous ID.
     */
    public IAuditID newInvocation() {
        IAuditID previous = threadLocal.get();
        AuditIDImpl id = new AuditIDImpl();
        id.generate();
        if (previous != null) {
            previous.increment();
            id.setParentID(previous.getLocalID());
        }
        threadLocal.set(id);
        return previous;
    }




}
